package bsoc.alpha;

import bsoc.nn.*;
import bsoc.util.*;

/**
* Initializes a Net for the Alpha-Integration.<p>
* Numbers of neurons per layer:<p>
* 40 (input layer), 25, 15, 10, 8 (output layer)<p>
* Nonnection probabilities between layers:<p>
* (1->0 : 50%)<p>
* (2->0 : 20%) (2->1 : 50%)<p>
* (3->0 : 100%) (3->1 : 10%) (3->0 : 5%)<p>
* (4->0 : 100%) (4->1 : 50%) (4->2 : 5%) (4->3 : 5%)<p>
*/ 

public class AlphaConnector extends NetConnector {

        public AlphaConnector (int seed) {
               super (seed, nodesPerLayer (), connProbMatrix (), new Weighter (0, 1),
                                                                                new Transfer (40));
        }

        static IntVector nodesPerLayer () {
                IntVector npl = new IntVector ();
                npl.addElement (40);
                npl.addElement (25);
                npl.addElement (15);
                npl.addElement (10);
                npl.addElement (8);
                return npl;
        }

        static IntVector connProbMatrix () {
                IntVector cpm = new IntVector ();
                IntVector cpl;

                        cpl = new IntVector ();
                cpm.addElement (cpl);
                        cpl = new IntVector ();
                        cpl.addElement (50);
                cpm.addElement (cpl);
                        cpl = new IntVector ();
                        cpl.addElement (20);
                        cpl.addElement (50);
                cpm.addElement (cpl);
                        cpl = new IntVector ();
                        cpl.addElement (5);
                        cpl.addElement (10);
                        cpl.addElement (100);
                cpm.addElement (cpl);
                        cpl = new IntVector ();
                        cpl.addElement (5);
                        cpl.addElement (5);
                        cpl.addElement (50);
                        cpl.addElement (100);
                cpm.addElement (cpl);

                return cpm;
        }


}


