package bsoc.division;

import java.util.*;
import java.io.*;
import bsoc.util.*;

/**
* Holds the results of the matches of a division. 
*/
class DTable extends SortableVector implements Serializable {

	void addEntry (DTableEntry e) {
		addElement (e);
	} 

	DTableEntry entryAt (int i) {
		return (DTableEntry)elementAt(i);
	} 

	DTableEntry entryForTeam (Team t) {
		int i;
		DTableEntry dte;

		for (i=0;i<size();i++) {
			dte = entryAt (i);
			if (dte.team == t) return dte;
		}
		throw new Error ("Team " + t + " not found in DTable" + this);
	}

	int teamIdForTeam (Team t) {
		return entryForTeam(t).getTeamId();		
	}

	DTableEntry entryForTeamId (int id) {
		int i;
		DTableEntry dte;

		for (i=0;i<size();i++) {
			dte = entryAt (i);
			if (dte.teamId == id) return dte;
		}
		throw new Error ("TeamId " + id + " not found in DTable" + this);
	}

	Team teamForTeamId (int id) {
		return entryForTeamId(id).getTeam();
	}


	void addMatchResult (Team teamA, Team teamB, int goalsTeamA, int goalsTeamB) {
		DTableEntry eA, eB;

		eA = entryForTeam (teamA);
		eB = entryForTeam (teamB);
		eA.addMatchResult (goalsTeamA, goalsTeamB);
		eB.addMatchResult (goalsTeamB, goalsTeamA);
	}

	Enumeration entries () {
		sort ();
		return elements();
	}	

	String toInfoString () {
		String str;
		Enumeration enum;
		DTableEntry e;

		str = DTableEntry.header () + "\n";
		enum = entries ();
		while (enum.hasMoreElements ()) {
			e = (DTableEntry)enum.nextElement ();
			str += e.toInfoString () + "\n";
		}
		return str;
	}

}

