package bsoc.division;

import java.util.*;
import java.io.*;
import bsoc.util.*;

/**
* Holds the results of one team of a division.
*/
public class DTableEntry extends Object implements bsoc.util.Comparable, Serializable {

	int matchesWon = 0;
	int matchesEven = 0;
	int matchesLost = 0; 
	int goalsGot = 0;
	int goalsShot = 0;
	int matchCount = 0;
	int points = 0;
	int teamId;
	Team team;

	public DTableEntry (Team t, int id) {
		team = t;
		teamId = id;
	}

	public boolean isGreaterThan (bsoc.util.Comparable o) {
		DTableEntry e;
		int v1, v2, d1, d2;

		e = (DTableEntry) o;
		v1 = getPoints ();
		v2 = e.getPoints ();
		if (v1 < v2) return true;
		else if (v1 == v2) {
			d1 = goalsShot - goalsGot;
			d2 = e.goalsShot - e.goalsGot;
			return (d1 < d2);
		}
		else return false;
	}

	public void addMatchResult (int myGoals, int theirGoals) {
		matchCount ++;
		goalsShot += myGoals;
		goalsGot += theirGoals;
		if (myGoals < theirGoals) matchesLost++;
		else if (myGoals == theirGoals) matchesEven++;
		else matchesWon++;
		calculatePoints ();
	}

	void calculatePoints () {
		points = matchesWon * 2 + matchesEven * 1;
	}

	public int getPoints () { return points; }
	public int getMatchesWon () { return matchesWon; }
	public int getMatchesEven () { return matchesEven; }
	public int getMatchesLost () { return matchesLost; }
	public int getGoalsGot () { return goalsGot; }
	public int getGoalsShot () { return goalsShot; }
	public int getMatchCount () { return matchCount; }
	public int getTeamId () { return teamId; }
	public Team getTeam () { return team; }

	static String header () {
		String str;

		str = "mWon\t";
		str += "mEven\t";
		str += "mLost\t";
		str += "gGot\t";
		str += "gShot\t";
		str += "mCount\t";
		str += "points\t";
		str += "teamId\t";
		str += "team";
		return str;
	}
		
	public String toInfoString () {
		String str;

		str = matchesWon + "\t";
		str += matchesEven + "\t";
		str += matchesLost + "\t";
		str += goalsGot + "\t";
		str += goalsShot + "\t";
		str += matchCount + "\t";
		str += points + "\t";
		str += teamId + "\t";
		str += team.toString();
		return str;
	}

        public boolean equals (Object o) {
 
                if (! (o instanceof DTableEntry)) return false;
                else {DTableEntry dte = (DTableEntry)o;
			if (
				(matchesWon == dte.matchesWon) &&
				(matchesEven == dte.matchesEven) &&
				(matchesLost == dte.matchesLost) &&
				(goalsGot == dte.goalsGot) &&
				(goalsShot == dte.goalsShot) &&
				(matchCount == dte.matchCount) &&
				(points == dte.points) &&
				(teamId == dte.teamId) &&
				(team.equals(dte.team))
			)
			return true;
			else return false;
		}
        }
}

