package bsoc.division;

import java.io.*;
import test.framework.*;
import bsoc.util.*;
import java.util.*;

/**
* All tests for the package bsoc.division
*/
public class DivisionTest extends BsocTestCase {

	Random ran;
        Division d, d1, d2, d3;

        DivisionTest (String s) {
                super (s);
        }
 
        public static Test suite () {
                TestSuite s = new TestSuite ();
                s.addTest (new DivisionTest ("testOneRound"));
                s.addTest (new DivisionTest ("testEquals1"));
                s.addTest (new DivisionTest ("testEquals2"));
                s.addTest (new DivisionTest ("testSer1"));
                s.addTest (new DivisionTest ("testSer2"));
                s.addTest (DTableEntryTest.suite ());
                s.addTest (SmallDiffDivisionTest.suite ());
                s.addTest (PlayerTest.suite ());
                s.addTest (TeamTest.suite ());
                return s;
        }

        protected void setUp () {
                ran = new Random (-1);
		d  = new TestDivision ();
		d1 = new TestDivision ();
		d2 = new TestDivision ();
		d3 = new TestDivision ();
        }

	public void testEquals1 () {
		assert (d1.equals(d1));
		assert (d1.equals(d2));
		assert (d1.equals(d3));
		assert (d2.equals(d1));
		assert (d2.equals(d2));
		assert (d2.equals(d3));
		assert (d3.equals(d1));
		assert (d3.equals(d2));
		assert (d3.equals(d3));
        }

	public synchronized void testEquals2 () {
		d1.start();
		d2.start();
		try {wait(100);}
		catch (InterruptedException e) {};
		assert (d1.equals(d2));
		assert (!d1.equals(d3));
        }

	public void testSer1 () throws IOException, ClassNotFoundException {
		Division d11;

		d11 = (Division)writeAndRead(d1);
		assert (d11.equals(d1));
		assert (d1.equals(d11));
        }

	public synchronized void testSer2 () throws IOException, ClassNotFoundException {
		Division d11;
		d1.start();
		try {wait(100);}
		catch (InterruptedException e) {};
		d11 = (Division)writeAndRead(d1);
		assert (d11.equals(d1));
		assert (d1.equals(d11));
        }

	public synchronized void testOneRound () {
                Enumeration enum;
                DTableEntry dte;

		d1.start(); 

		try {wait(100);}
		catch (InterruptedException e) {};

                enum = d1.dTableEntries ();
                dte = (DTableEntry)enum.nextElement ();

                assert (dte.getMatchesWon () == 19);
                assert (dte.getMatchesEven () == 7);
                assert (dte.getMatchesLost () == 14);
                assert (dte.getGoalsGot () == 166);
                assert (dte.getGoalsShot () == 192);
                assert (dte.getMatchCount () == 40);
                assert (dte.getPoints () == 45);
	}


}
