package bsoc.genetic;

import java.util.*;

/**
* A Crossover switch which generates a boolean that switches from true to false
* after random intervals. The interval length is normal distributed. 
*/
public class CrossoverSwitch extends Object {

	Random ran;
	int mean;
	int var;
	int nextSwitchAt;
	int count;
	boolean takeA = true;

	/**
	* @param mean	The mean length of an interval.
	* @param var	The variance of the length of the interval.
	* @param seed	Initial Value for the random generator. 
	*/
	public CrossoverSwitch (int mean, int var, long seed) {
		ran = new Random (seed);
		this.mean = mean;	
		this.var = var;
		count = 0;
		nextSwitchAt = setNextSwitchAt (mean, var, ran);
	}

	/**
	* Undefined initialisation of the random generator.
	*/
	public CrossoverSwitch (int mean, int var) {
		ran = new Random ();
		this.mean = mean;	
		this.var = var;
		count = 0;
		nextSwitchAt = setNextSwitchAt (mean, var, ran);
	}

	static int setNextSwitchAt (int mean, int var, Random ran) {
		return mean + (int)(ran.nextGaussian() * var);
	}

	int setNextSwitchAt () {
		return setNextSwitchAt (mean, var, ran);
	}	
	
	/**
	* Returns the next value of the boolean sequence. Can be used to determine if a
	* mutation has to take place or not.
	*/
	public boolean takeA () {
		if (count >= nextSwitchAt) {
			count = 0;
			nextSwitchAt = setNextSwitchAt();
			takeA = !takeA;
		}
		count++;
		return takeA; 
	}	 		

}	

