package bsoc.genetic;

import java.util.*;

/**
* Generates a serie of booleans. The probability of true is the mutation rate. 
*/
public class Mutator extends Object {

	Random ran;
	int mutationRate;

	/**
	* Generates a mutator that gives a serie of booleans where
	* the probabillity of true is the mutation rate.
	*
	* @param mutRate	The mutation rate in per million. Values must be
	*			between 0 and 1000000.
	* @param seed		Initial Value for the random generator. 
	*/
	public Mutator (int mutRate, long seed) {
		ran = new Random (seed);
		setMutationRate (mutRate); 
	}

	/**
	* Undefined initialisation of the random generator.
	*/
	public Mutator (int mutRate) {
		ran = new Random ();
		setMutationRate (mutRate); 
	}

	/**
	* Returns the next value of the boolean sequence. Ca be used to determine if a
	* mutation has to take place or not.
	*/
	public boolean isMutation () {
		return ((Math.abs(ran.nextInt()) % 1000000) < mutationRate);
	}

	void setMutationRate (int mutRate) {
		if (mutRate < 0) throw new Error("Mutation rate must be greater than 0");
		if (mutRate > 1000000) throw new Error("Mutation rate must be smaller than 1000000");
		mutationRate = mutRate;
	}

}	

