package bsoc.nn;

import java.util.*;
import java.io.*; 

/**
* A set of layer nodes.
* @see LayerNode
*/
public class Layer implements Serializable {

	Vector lns;

	Layer () {
		lns = new Vector ();
	}	

	Layer (int size) {
		int i;

		lns = new Vector ();
		for (i=1; i<=size; i++) {
			this.addLayerNode (new LayerNode ());
		}
	}

	int size () {
		return lns.size ();
	}

	LayerNode layerNodeAt (int i) {
		return (LayerNode)lns.elementAt (i);
	}

	void addLayerNode (LayerNode ln) {
		this.lns.addElement (ln);
	}

	public void setValueAt (int i, short value) {
		LayerNode ln; 
		ln = this.layerNodeAt (i);
		ln.setValue (value);
	} 

	public short getValueAt (int i) {
		LayerNode ln; 
		ln = this.layerNodeAt (i);
		return (ln.getValue ());
	} 

	void setValuesRandom (RandomValue rv) {
		LayerNode ln;
		Enumeration e;

		for (e = layerNodes (); e.hasMoreElements ();) {
			ln = (LayerNode)e.nextElement ();
			ln.setValueRandom (rv);
		}
	}

	public String toString () {
		String str; 
		Enumeration e;
		LayerNode ln;

		str = "[";
		for (e = layerNodes (); e.hasMoreElements ();) {
			ln = (LayerNode)e.nextElement ();
			str = str + ln.toString ();
		}
		str = str + "]";
		return str;
	}

	String valuesToString () {
		String str; 
		Enumeration e;
		LayerNode ln;

		str = "";
		for (e = layerNodes (); e.hasMoreElements ();) {
			ln = (LayerNode)e.nextElement ();
			str = str + ln.valueToString () + ";";
		}
		return str;
	}



	void resetCalculated () {
		int i, size;
		LayerNode ln;
		
		size = size ();
		for (i=0; i<size; i++) {
			ln = layerNodeAt (i);
			ln.resetCalculated ();
		}
	}

	void calculate () {
		int i, size;
		LayerNode ln;
		
		size = size ();
		for (i=0; i<size; i++) {
			ln = layerNodeAt (i);
			if (! ln.isCalculated ()) {
				ln.calculate ();
			}
		}
	}
		
	public Enumeration layerNodes () {
		return new EnumLayerNodesOfLayer (this);
	}	

        public boolean equals (Object o) {
                return equalsInValues (o);
        }

        public boolean equalsInValues (Object o) {
                LayerNode ln, thisLn;
                Enumeration lns, thisLns;
                boolean equals = true;
                Layer l;
 
                if (! (o instanceof Layer)) return false;
                else {
                        l = (Layer)o;
                        lns = l.layerNodes ();
                        thisLns = layerNodes ();
                        while (lns.hasMoreElements () && thisLns.hasMoreElements () && equals) {
                                ln = (LayerNode)lns.nextElement ();
                                thisLn = (LayerNode)thisLns.nextElement ();
                                if (! ln.equalsInValue (thisLn)) equals = false;
                        }
                        if (equals == true) {
                                if (thisLns.hasMoreElements () || lns.hasMoreElements ()) return false;
                                else return true;
                        }
                        else return false;
                }
        }

} 

class EnumLayerNodesOfLayer implements Enumeration {

	Vector v;
	int i, size;

	EnumLayerNodesOfLayer (Layer l) {
		v =  l.lns;
		i = 0;
		size = v.size ();
	}
	
	public boolean hasMoreElements () {
		if (i >= size) return false;
		else return true;
	}

	public Object nextElement () {
		Object o;

		o = v.elementAt (i);
		i++;
		return o;
	}
}
