package bsoc.nn;

import java.io.*;
import java.util.*;
import bsoc.util.*;
import bsoc.genetic.*;
import test.framework.*;

/**
* Testcases.
*/
public class NetTest extends TestCase {

        static NetConnector nc = new NetConnector ();
        static Weighter wgt = new Weighter ();
        static Transfer trans = new Transfer ();

        LayerNode ln1, ln2, ln3;
        Neuron neu1, neu2, neu3;
        NeuronLayer nl1, nl2, nl3;
        Net n1, n2, n3, n4;
        RandomValue rv;
        RandomWgt rw;

        NetTest (String s) {
                super (s);
        }
 
        public static Test suite () {
                TestSuite s = new TestSuite ();
                s.addTest (new NetTest ("testCrossoverA"));
                s.addTest (new NetTest ("testEqualsLayerNode"));
                s.addTest (new NetTest ("testEqualsNeuron"));
                s.addTest (new NetTest ("testEqualsNeuronLayer"));
                s.addTest (new NetTest ("testEqualsNet"));
                s.addTest (new NetTest ("testCalculate"));
                s.addTest (new NetTest ("testSerialize"));
//              s.addTest (new NetTest ("testValues"));
                return s;
        }

        protected void setUp () {

                rv = new RandomValue (293847);
                rw = new RandomWgt (23847);

                ln1 = new LayerNode ((short) 2); 
                ln2 = new LayerNode ((short) 2);
                ln3 = new LayerNode ((short) 4);

                neu1 = new Neuron (trans);
                neu1.setValue ((short)3);
                generateSynapsesForNeuron (neu1);
                neu2 = new Neuron (trans);
                neu2.setValue ((short)3);
                generateSynapsesForNeuron (neu2);
                neu3 = new Neuron (trans);
                neu3.setValue ((short)3);
                generateSynapsesForNeuron (neu3);

                nl1 = new NeuronLayer ();
                nl1.addNeuron (new Neuron (trans));
                nl1.addNeuron (new Neuron (trans));
                nl1.addNeuron (new Neuron (trans));
                nl1.addNeuron (new Neuron (trans));
                nl1.addNeuron (new Neuron (trans));
                generateSynapsesForNeuronLayer (nl1);
                nl2 = new NeuronLayer ();
                nl2.addNeuron (new Neuron (trans));
                nl2.addNeuron (new Neuron (trans));
                nl2.addNeuron (new Neuron (trans));
                nl2.addNeuron (new Neuron (trans));
                nl2.addNeuron (new Neuron (trans));
                generateSynapsesForNeuronLayer (nl2);
                nl3 = new NeuronLayer ();
                nl3.addNeuron (new Neuron (trans));
                nl3.addNeuron (new Neuron (trans));
                nl3.addNeuron (new Neuron (trans));
                nl3.addNeuron (new Neuron (trans));
                nl3.addNeuron (new Neuron (trans));
                generateSynapsesForNeuronLayer (nl3);

                nc = new NetConnector ();
                n1 = new Net (nc);
                n1.setWeightsRandom (new RandomWgt (-1));
                n2 = new Net (nc);
                n2.setWeightsRandom (new RandomWgt (-1));
        }

        void generateSynapsesForNeuron (Neuron neu) {
                int i;
                Synapse syn;
                Weighter w = new Weighter ();
 
                syn = new Synapse (w);
                syn.setWeight ((short)-1);
                neu.addSynapse (syn);

                syn = new Synapse (w);
                syn.setWeight ((short)3);
                neu.addSynapse (syn);

                syn = new Synapse (w);
                syn.setWeight ((short)1);
                neu.addSynapse (syn);

                syn = new Synapse (w);
                syn.setWeight ((short)4);
                neu.addSynapse (syn);

        }

        void generateSynapsesForNeuronLayer (NeuronLayer nl) {
                Enumeration neus;
                Neuron neu;

                neus = nl.layerNodes ();
                while (neus.hasMoreElements ()) {
                        neu = (Neuron)neus.nextElement ();
                        generateSynapsesForNeuron (neu);
                }
        }

        public void testCrossoverA () {
		Net n1, n2, n3;
		CrossoverSwitch cs = new CrossoverSwitch(5, 2);
		Mutator mut = new Mutator (300000); // 30%
		Enumeration enum1, enum2, enum3;	
		Synapse s1, s2, s3;
		RandomWgt rw = new RandomWgt();
		int from1Count, from2Count, fromNoneCount;
		
		n1 = new Net(nc);
		n2 = new Net(nc);
		n1.setWeightsRandom(rw);
		n2.setWeightsRandom(rw);
		n3 = (Net)n1.newChild(n2, cs, mut);

		enum1 = n1.synapses();
		enum2 = n2.synapses();
		enum3 = n3.synapses();
		from1Count = 0;
		from2Count = 0;
		fromNoneCount = 0;
		while (enum1.hasMoreElements()) {
			s1 = (Synapse)enum1.nextElement();
			s2 = (Synapse)enum2.nextElement();
			s3 = (Synapse)enum3.nextElement();
			if (s1.getWeight() == s3.getWeight()) from1Count++;
			else if (s2.getWeight() == s3.getWeight()) from2Count++;
			else fromNoneCount++;
		}		
		assert (from1Count > 0);
		assert (from2Count > 0);
		assert (fromNoneCount > 0);

	}

        public void testEqualsLayerNode () {
                assert (ln1.equals (ln1));
                assert (ln1.equals (ln2));
                assert (ln2.equals (ln1));
                assert (! ln1.equals (ln3));
                assert (! ln2.equals (ln3));

                assert (ln1.equalsInValue (ln1));
                assert (ln1.equalsInValue (ln2));
                assert (ln2.equalsInValue (ln1));
                assert (! ln1.equalsInValue (ln3));
                assert (! ln2.equalsInValue (ln3));
        }


        public void testEqualsNeuron () {
                Synapse syn;

                assert (neu1.equals (neu2));
                assert (neu1.equals (neu3));
                assert (neu1.equals (neu1));
                assert (neu3.equals (neu2));

                assert (neu1.equalsInValue (neu2));
                assert (neu1.equalsInValue (neu3));
                assert (neu1.equalsInValue (neu1));
                assert (neu3.equalsInValue (neu2));

                assert (neu1.equalsInWeights (neu2));
                assert (neu1.equalsInWeights (neu3));
                assert (neu1.equalsInWeights (neu1));
                assert (neu3.equalsInWeights (neu2));

                neu2.setValue ((short) 4);
                assert (! neu1.equals (neu2));
                assert (neu1.equals (neu3));
                assert (neu1.equals (neu1));
                assert (! neu3.equals (neu2));

                assert (! neu1.equalsInValue (neu2));
                assert (neu1.equalsInValue (neu3));
                assert (neu1.equalsInValue (neu1));
                assert (! neu3.equalsInValue (neu2));

                assert (neu1.equalsInWeights (neu2));
                assert (neu1.equalsInWeights (neu3));
                assert (neu1.equalsInWeights (neu1));
                assert (neu3.equalsInWeights (neu2));
 
                syn = neu2.synapseAt (1);
                syn.setWeight ((short)-10);
                assert (! neu1.equals (neu2));
                assert (neu1.equals (neu3));
                assert (neu1.equals (neu1));
                assert (! neu3.equals (neu2));

                assert (! neu1.equalsInValue (neu2));
                assert (neu1.equalsInValue (neu3));
                assert (neu1.equalsInValue (neu1));
                assert (! neu3.equalsInValue (neu2));

                assert (! neu1.equalsInWeights (neu2));
                assert (neu1.equalsInWeights (neu3));
                assert (neu1.equalsInWeights (neu1));
                assert (! neu3.equalsInWeights (neu2));

        }


        public void testEqualsNeuronLayer () {
                Synapse syn;
                Neuron neu;

                assert (nl1.equals (nl2));
                assert (nl1.equals (nl3));
                assert (nl1.equals (nl1));
                assert (nl3.equals (nl2));

                assert (nl1.equalsInValues (nl2));
                assert (nl1.equalsInValues (nl3));
                assert (nl1.equalsInValues (nl1));
                assert (nl3.equalsInValues (nl2));

                assert (nl1.equalsInWeights (nl2));
                assert (nl1.equalsInWeights (nl3));
                assert (nl1.equalsInWeights (nl1));
                assert (nl3.equalsInWeights (nl2));

                neu = nl2.neuronAt (1);
                neu.setValue ((short) 4);
                assert (! nl1.equals (nl2));
                assert (nl1.equals (nl3));
                assert (nl1.equals (nl1));
                assert (! nl3.equals (nl2));

                assert (! nl1.equalsInValues (nl2));
                assert (nl1.equalsInValues (nl3));
                assert (nl1.equalsInValues (nl1));
                assert (! nl3.equalsInValues (nl2));

                assert (nl1.equalsInWeights (nl2));
                assert (nl1.equalsInWeights (nl3));
                assert (nl1.equalsInWeights (nl1));
                assert (nl3.equalsInWeights (nl2));
 
                neu = nl2.neuronAt (1);
                syn = neu.synapseAt (1);
                syn.setWeight ((short)-10);
                assert (! nl1.equals (nl2));
                assert (nl1.equals (nl3));
                assert (nl1.equals (nl1));
                assert (! nl3.equals (nl2));

                assert (! nl1.equalsInValues (nl2));
                assert (nl1.equalsInValues (nl3));
                assert (nl1.equalsInValues (nl1));
                assert (! nl3.equalsInValues (nl2));

                assert (! nl1.equalsInWeights (nl2));
                assert (nl1.equalsInWeights (nl3));
                assert (nl1.equalsInWeights (nl1));
                assert (! nl3.equalsInWeights (nl2));

        }


        public void testEqualsNet () throws IOException {
                Enumeration syns;
                Synapse syn = new Synapse (wgt);
                Neuron neu;
                NeuronLayer nl;
                int i;
 
                assert (n1.equals (n1));
                assert (n1.equalsInValues (n1));
                assert (n1.equalsInWeights (n1));
                assert (n2.equals (n1));
                assert (n2.equalsInValues (n1));
                assert (n2.equalsInWeights (n1));

                nl = (NeuronLayer)n2.layerAt (2);
                neu = (Neuron)nl.layerNodeAt (1);
                neu.setValueRandom (rv);
                assert (! n1.equals (n2));
                assert (! n1.equalsInValues (n2));
                assert (n1.equalsInWeights (n2));

                syns = n2.synapses ();
                for (i=0; i<4; i++) {
                        syn = (Synapse)syns.nextElement ();
                }
                syn.setWeightRandom (rw);
                assert (! n1.equals (n2));
                assert (! n1.equalsInValues (n2));
                assert (! n1.equalsInWeights (n2));

        }
 
        public void testCalculate () {
                Enumeration syns;
                Synapse syn = new Synapse (wgt);
                Layer il1, ol1, il2, ol2;
                int i;

                il1 = n1.getInputLayer ();
                ol1 = n1.getOutputLayer ();
                il2 = n2.getInputLayer ();
                ol2 = n2.getOutputLayer ();
 
                assert (il1.equals (il2));
                il1.setValuesRandom (new RandomValue (7236));
                il2.setValuesRandom (new RandomValue (7236));
                assert (il1.equals (il2));

                assert (ol1.equals (ol2));
                n1.recalculate ();
                assert (! ol1.equals (ol2));
                n2.recalculate ();
                assert (ol1.equals (ol2));

                il2.setValuesRandom (new RandomValue (736));
                n2.recalculate ();
                assert (! ol1.equals (ol2));
 
                il2.setValuesRandom (new RandomValue (7236));
                n2.recalculate ();
                assert (ol1.equals (ol2));
 
                syns = n2.synapses ();
                for (i=0; i<4; i++) {
                        syn = (Synapse)syns.nextElement ();
                }
                syn.setWeightRandom (rw);
                n2.resetCalculated ();
                il2.setValuesRandom (new RandomValue (7236));
                n2.calculate ();
                assert (! ol1.equals (ol2));

        }


	public void testSerialize () throws IOException, ClassNotFoundException {
                Net n11;

		FileOutputStream ostream;
		FileInputStream istream;
		ObjectOutputStream po;
		ObjectInputStream pi;
		String file = "Net.object";

		n1.setWeightsRandom (new RandomWgt (234));
                n11 = new Net (nc);
		n11.setWeightsRandom (new RandomWgt (82736));
                assert (! n11.equalsInWeights (n1));

		ostream = new FileOutputStream(file);
                po = new ObjectOutputStream(ostream);
		po.writeObject (n1);
		po.flush();       
		ostream.close();
 
		istream = new FileInputStream(file);
		pi = new ObjectInputStream(istream);
		n11 = (Net)pi.readObject();
		istream.close();

                assert (n11.equalsInWeights (n1));
	}

	void runGeneral () {
		Net net = new Net (nc);

		net.setWeightsRandom (new RandomWgt ());
		net.setInputLayerValuesRandom (new RandomValue ());
		net.calculate ();
	}

	void runConnStat () throws IOException {
		FileOutputStream os;
		PrintWriter ps;
		Net net = new Net (nc);
		String file = "ConnStat.txt";
	
		os = new FileOutputStream (file);
		ps = new PrintWriter (os);
		net.writeConnStatToStream (ps);
		os.close ();
		System.out.println ("--- Connection Statistics: Results in "+file);
	}
			
	void runSynStat () throws IOException {
		FileOutputStream os; 
		PrintWriter ps; 
		Net net = new Net (nc);
		String file = "SynStat.txt";
	 
		os = new FileOutputStream (file); 
		ps = new PrintWriter (os); 
		net.writeSynStatToStream (ps);
		os.close (); 
		System.out.println ("--- Synapses Statistics : Results in "+file);
	} 
			 
	void runCrossover () {
		int i;
		Net net, netFather, netMother;
		String file = "Crossover.txt";
		Random r = new Random ();
		net = new Net (nc);
		net.setWeightsRandom (new RandomWgt ());
		netFather = new Net (nc);
		netFather.setWeightsRandom (new RandomWgt ());
		netMother = new Net (nc);
		netMother.setWeightsRandom (new RandomWgt ());
		net = new Net (nc);
		net.setWeightsCrossover (netMother, netFather, r);
		for (i=1; i<=30; i++) {
			netFather.setWeightsRandom (new RandomWgt ()); 
			netMother.setWeightsRandom (new RandomWgt ()); 
			net.setWeightsCrossover (netMother, netFather, r); 
		}	
	}


}


