package bsoc.nn;

import java.util.*;
import java.io.*; 

/**
* A neuron. Holds an activation level and is connected with other neurons or layer nodes
* via synaptic connections.
**/

class Neuron extends LayerNode implements Serializable {
	
	Transfer transfer; 

	Vector syns;
	boolean isCalculated;

	Neuron (Transfer trans) {
                transfer = trans;
		isCalculated = false;
		syns = new Vector ();
		setValue ((short)0);
	}

	void setCalculated () {
		isCalculated = true;
	}
	void resetCalculated () {
		isCalculated = false;
	}

	boolean isCalculated () {
		return isCalculated;
	}

	void recalculate () {
                resetCalculated ();
                calculate ();
	}

	void calculate () {
		int i, size, wval, sum;
		Synapse syn;
		LayerNode ln;
		
		if (! isCalculated ()) {
			size = synapseCount ();
			sum = 0;
			for (i=0; i<size; i++) {
				syn = synapseAt (i);
				wval = syn.getWeightedCalculatedValue ();
				sum = sum + wval;
			}
			setValue (transfer.getValue (sum));
			setCalculated ();
		}	
	}

	void addSynapse (Synapse syn) {
		this.syns.addElement (syn);
	}

	int synapseCount () {
		return this.syns.size ();
	}

	Synapse synapseAt (int i) {
		return (Synapse)this.syns.elementAt (i);
	}

	void setWeightsRandom (RandomWgt rw) {
		int i, size;
		Synapse syn;

		size = synapseCount ();
		for (i=0; i<size; i++) {
			syn = synapseAt (i);
			syn.setWeightRandom (rw);
		}
	}

	public String toString () {
		String str;
		int size = this.synapseCount ();
		Synapse syn; 
		int i;

		str = super.toString ();
                if (transfer == null) {
                        str += "*** No Transfer ***";
                }
		str += "{";
		for (i=0; i < size; i++) {
			syn = this.synapseAt (i);
			str = str + syn.toString ();
		}
		str = str + "}";
		return str;
	}

	boolean isConnectedToLayerNode (LayerNode ln) {
		int i, size;
		boolean is = false;
		Synapse syn;
		
		size = synapseCount ();
		for (i=0; i<size; i++) {
			syn = synapseAt (i);
			if (syn.layerNode () == ln) {
				is = true;
				break;
			}
		}
		return is;
	}

	Enumeration synapses () {
		return new EnumSynapsesOfNeuron (this); 
	}

	void readObject(java.io.ObjectInputStream stream) throws IOException, ClassNotFoundException {
                super.readObject (stream);
		syns = (Vector)stream.readObject ();	
	} 

	void writeObject(java.io.ObjectOutputStream stream) throws IOException {
                super.writeObject (stream);
		stream.writeObject (syns);
	}

        public boolean equalsInWeights (Object o) {
                Synapse syn, thisSyn;
                Enumeration syns, thisSyns;
                boolean equals = true;
                Neuron neu;
 
                if (! (o instanceof Neuron)) return false;
                else {
                        neu = (Neuron)o;
                        syns = neu.synapses ();
                        thisSyns = synapses ();
                        while (syns.hasMoreElements () && thisSyns.hasMoreElements () && equals) {
                                syn = (Synapse)syns.nextElement ();
                                thisSyn = (Synapse)thisSyns.nextElement ();
                                if (! syn.equals (thisSyn)) equals = false;
                        }
                        if (equals == true) {
                                if (thisSyns.hasMoreElements () || syns.hasMoreElements ()) return false;
                                else return true;
                        }
                        else return false;
                }
        }

        public boolean equals (Object o) {
                return (equalsInValue (o) && equalsInWeights (o));
        }
 
}

class EnumSynapsesOfNeuron implements Enumeration {

	Vector syns;
	int i, size;

	EnumSynapsesOfNeuron (Neuron neu) {
		syns =  neu.syns;
		i = 0;
		size = syns.size ();
	}

	
	public boolean hasMoreElements () {
		if (i >= size) return false;
		else return true;
	}

	public Object nextElement () {
		Object syn;

		syn = syns.elementAt (i);
		i++;
		return syn;
	}
}
