package bsoc.nn;

import java.util.*;
import java.io.*; 

/**
* A layer of a ANN containing a collection of neurons.
* @see Neuron
*/

class NeuronLayer extends Layer implements Serializable {
	

        void addNeuron (Neuron neu) {
                addLayerNode (neu);
        }
 
        Neuron neuronAt (int i) {
                return (Neuron)layerNodeAt (i);
        }
 
	void setWeightsRandomOld (RandomWgt rw) {
		int i, size;
		Neuron neu;

		size = size ();
		for (i=0; i<size; i++) {
			neu = (Neuron)layerNodeAt (i);
			neu.setWeightsRandom (rw);
		}
	}

	void setWeightsRandom (RandomWgt rw) {
		Enumeration e;
		Synapse syn;

		for (e = synapses (); e.hasMoreElements ();) {
			syn = (Synapse)e.nextElement ();
			syn.setWeightRandom (rw);
		}
	}

        public boolean equals (Object o) {
                return equalsInWeights (o) && equalsInValues (o);
        }

        public boolean equalsInWeights (Object o) {
                Neuron neu, thisNeu;
                Enumeration neus, thisNeus;
                boolean equals = true;
                NeuronLayer nl;
 
                if (! (o instanceof NeuronLayer)) return false;
                else {
                        nl = (NeuronLayer)o;
                        neus = nl.layerNodes ();
                        thisNeus = layerNodes ();
                        while (neus.hasMoreElements () && thisNeus.hasMoreElements () && equals) {
                                neu = (Neuron)neus.nextElement ();
                                thisNeu = (Neuron)thisNeus.nextElement ();
                                if (! neu.equalsInWeights (thisNeu)) equals = false;
                        }
                        if (equals == true) {
                                if (thisNeus.hasMoreElements () || neus.hasMoreElements ()) return false;
                                else return true;
                        }
                        else return false;
                }
        }

	Enumeration synapses () {
		return new EnumSynapsesOfNeuronLayer (this);
	}	
}

class EnumSynapsesOfNeuronLayer implements Enumeration {

	Enumeration en, es;
	Object nextSyn = null;

	EnumSynapsesOfNeuronLayer (NeuronLayer nl) {
		Neuron neu;

		en = nl.layerNodes ();
		if (en.hasMoreElements ()) {
			neu  = (Neuron)en.nextElement ();
			es = neu.synapses (); 
			nextSyn = nextSyn ();
		}
	}


	Object nextSyn () {
		Neuron neu;

		if ( es.hasMoreElements ()) return es.nextElement ();
		else if (en.hasMoreElements ()) {
			neu = (Neuron)en.nextElement (); 
			es = neu.synapses (); 
			return nextElement(); 
		}
		else return null;
	}

	public boolean hasMoreElements () { 
		if (nextSyn == null) return false;
		else return true;
	}

	public Object nextElement () {
		Object o;

		o = nextSyn;
		nextSyn = nextSyn ();
		return o;
	}
}

