package bsoc.nn;
 
import java.util.*;
import java.io.*;

/**
* Implements the transfer function for neurons. The values are calculated at
* initialization time and stored in an array. If a value has to be calculated
* it is taken from this array.
*/
public class Transfer implements Serializable {
	Vector vals = new Vector ();
	int maxIndex;
	short maxValue;

	public Transfer () {
                initTransfer (Params.maxValue, 50);
	}

	public Transfer (int stretch) {
                initTransfer (Params.maxValue, stretch);
	}

	void initTransfer (short maxValue, int stretch) {
		int i;
		Integer val;
		double dVal;
		
		this.maxValue = maxValue;
		initMaxIndex (stretch);
		for (i=0; i<maxIndex; i++) {
			dVal = 1 / (1 + Math.exp (-(i + 1) / (double)stretch)) * (maxValue - 1);
			val = new Integer ((int)dVal + 1); 
			vals.addElement (val);
		}
	}

	void initMaxIndex (int stretch) {
		double x, y;
		
		y = 1 - (1.0 / (maxValue * 2));
		x = - (Math.log (1/y - 1) * (double)stretch);
		maxIndex = (int)x;
	}

	short getValue (int in) {
		Integer val;

		if (in >= 0) {
			if (in >= maxIndex) { 
				return (short)(maxValue - 1);
			}
			else {
				val = (Integer)vals.elementAt (in);
				return (short)val.intValue ();
			}
		}
		else {
			if (-in >= maxIndex) {  
				return (short)0; 
			} 
			else { 
				val = (Integer)vals.elementAt (-in); 
				return (short)(maxValue - (short)val.intValue ()); 
			} 
		}
			
	}	

	void writeToStream (PrintWriter str) {
		short result;
		int val, from, to;

		from = (int)(- maxIndex * 1.1); 
		to = (int)(maxIndex * 1.1);	
		
		str.println ("val;result");
		for (val=from; val<=to; val++) {
			result = getValue (val);
			str.println (val+";"+result);
 		}
	}	
}

