package bsoc.socsim;

import bsoc.util.*;
import java.lang.reflect.*;
import java.io.*;
import java.util.*;
import java.sql.*;
import java.awt.*;
import java.awt.event.*;
import java.lang.reflect.*;
import EDU.gatech.cc.is.abstractrobot.*;
import EDU.gatech.cc.is.simulation.*;
import EDU.gatech.cc.is.util.*;
import JavaSoccer.teams.*;

/**
* Soccer simulation that may run abstract robots from JavaBot.
*/

public class SocSim extends Thread {

	public static int CHANGED_FOR_INFO = 0;
	public static int CHANGED_FOR_GRAPHICS = 1;
	public static int STOPPED = 2;

	private int notificationIntervalInfo = 50;
	private int notificationIntervalGraphics = 1;

	private SimulatedObject simulatedObjects[] = new SimulatedObject [16];
	private ControlSystemS controlSystems[] = new ControlSystemS [10];

	private double top, bottom, left, right;

	private long simulationSteps;
	private long currentTime;
	private long lastTime;
	private long simTimestep;
	private int eastscore;
	private int westscore;
	private int seed = 3298;

	private boolean matchStopped = true;
	private boolean simulationRunning = true;

	private boolean robotsDone = true;

        private Vector simulationListener = new Vector ();
	private	Color nullColor = new Color (0);


        public SocSim () {
		setPriority(MIN_PRIORITY);
		initValues();
		addAndInitSimulatedObjects();
		initRobotsAtStartup();
        }

	public void initRobotsAtStartup() {
		addAndInitRobots();
		setupSimulatedObjects();
	}

	int indexSocField()			{return 0;}
	int indexCornerPanel1()			{return 1;}
	int indexCornerPanel2()			{return 2;}
	int indexCornerPanel3()			{return 3;}
	int indexCornerPanel4()			{return 4;}
	int indexBall()				{return 5;}

	int baseIndexRobots()			{return 6;}
	int baseIndexControlSystems()		{return 0;}

	int baseIndexRobotsWest()		{return baseIndexRobots()+0;}
	int baseIndexRobotsEast()		{return baseIndexRobots()+5;}

	int baseIndexControlSystemsWest()	{return baseIndexControlSystems()+0;}
	int baseIndexControlSystemsEast()	{return baseIndexControlSystems()+5;}

	

        public void initValues () {
		top = 0.8625;
		bottom = -0.8625;
		left = -1.47;
		right = 1.47;
		simulationSteps = 0;
		eastscore = 0;
		westscore = 0;
		lastTime = System.currentTimeMillis();
		currentTime = System.currentTimeMillis();
        }

	/**
	* Initializes all objects that are needed beside robots.
	*/
        public void addAndInitSimulatedObjects () {

		simulatedObjects[indexSocField()] = new SocFieldSmallSim ();
		initSocField();

		simulatedObjects[indexCornerPanel1()] = new ObstacleInvisibleSim ();
		initCornerPanel1 ();

		simulatedObjects[indexCornerPanel2()] = new ObstacleInvisibleSim ();
		initCornerPanel2 ();

		simulatedObjects[indexCornerPanel3()] = new ObstacleInvisibleSim ();
		initCornerPanel3 ();

		simulatedObjects[indexCornerPanel4()] = new ObstacleInvisibleSim ();
		initCornerPanel4 ();

		simulatedObjects[indexBall()] = new GolfBallNoiseSim ();
		initBall();
        }

	void initSocField () {
		simulatedObjects[indexSocField()].init 
			(0, 0, 0, 0, nullColor, nullColor, 0, indexSocField(), seed ++);
	}

	void initCornerPanel1 () {
		simulatedObjects[indexCornerPanel1()].init 
			(2.047, 1.4396, 0, 1.0, nullColor, nullColor, 0, indexCornerPanel1(), seed ++);
	}

	void initCornerPanel2 () {
		simulatedObjects[indexCornerPanel2()].init 
			(-2.047, 1.4396, 0, 1.0, nullColor, nullColor, 0, indexCornerPanel2(), seed ++);
	}

	void initCornerPanel3 () {
		simulatedObjects[indexCornerPanel3()].init 
			(2.047, -1.4396, 0, 1.0, nullColor, nullColor, 0, indexCornerPanel3(), seed ++);
	}

	void initCornerPanel4 () {
		simulatedObjects[indexCornerPanel4()].init 
			(-2.047, -1.4396, 0, 1.0, nullColor, nullColor, 0, indexCornerPanel4(), seed ++);
	}

	void initBall () {
		simulatedObjects[indexBall()].init 
			(0, 0, 0, 0.02, Color.orange, nullColor, 3, indexBall(), seed ++);
	}

	/**
	* Should be overridden by subclasses.
	*/
        public void addAndInitRobots () {

//---------- West Team ----------------------------------------------------------------------------

		addAndInitRobotWest (new Kechze(), 0);
		addAndInitRobotWest (new Kechze(), 1);
		addAndInitRobotWest (new Kechze(), 2);
		addAndInitRobotWest (new Kechze(), 3);
		addAndInitRobotWest (new Kechze(), 4);

//---------- East Team ----------------------------------------------------------------------------

		addAndInitRobotEast (new SchemaDemo(), 0);
		addAndInitRobotEast (new SchemaDemo(), 1);
		addAndInitRobotEast (new SchemaDemo(), 2);
		addAndInitRobotEast (new SchemaDemo(), 3);
		addAndInitRobotEast (new SchemaDemo(), 4);
        }

	/**
	* Should be used in addAndInitRobots().
	*/
	public void addAndInitRobotWest (ControlSystemS cs, int nr) {

		simulatedObjects[baseIndexRobotsWest()+nr] = new SocSmallSim ();
		controlSystems[baseIndexControlSystemsWest()+nr] = cs;
		initRobotWest (nr);
	}

	void initRobotWest (int nr) {
		controlSystems[baseIndexControlSystemsWest()+nr].init 
			((Simple)simulatedObjects[baseIndexRobotsWest()+nr], seed++);
		simulatedObjects[baseIndexRobotsWest()+nr].init 
			(-1, 0, 0, 0, Color.red, Color.yellow, 1, baseIndexRobotsWest()+nr, seed++);
	}

	/**
	* Should be used in addAndInitRobots().
	*/
	public void addAndInitRobotEast (ControlSystemS cs, int nr) {

		simulatedObjects[baseIndexRobotsEast()+nr] = new SocSmallSim ();
		controlSystems[baseIndexControlSystemsEast()+nr] = cs;
		initRobotEast (nr);
	}

	void initRobotEast (int nr) {
		controlSystems[baseIndexControlSystemsEast()+nr].init 
			((Simple)simulatedObjects[baseIndexRobotsEast()+nr], seed++);
		simulatedObjects[baseIndexRobotsEast()+nr].init 
			(1, 0, 0, 0, Color.blue, Color.white, 2, baseIndexRobotsEast()+nr, seed++);
	}

	/**
	* Must always be called after new Robots are set to the simulation.
	*/
	public void setupSimulatedObjects () {
		SimulatedObject simObj;
		ControlSystemS contSys;
		int i;

		for(i=0; i<simulatedObjects.length; i++) {	
			// let everyone take a step to update their pointers
                        simObj = simulatedObjects [i];
			simObj.takeStep(0,simulatedObjects);
		}
		for(i=0; i<controlSystems.length; i++) {
			contSys = controlSystems[i];
			contSys.configure();
		}
	}

	public void setNotificationIntervalInfo (int val) {
		if (val < 0) notificationIntervalInfo = 0;
		else notificationIntervalInfo = val;
	}

	public void setNotificationIntervalGraphics (int val) {
		if (val < 0) notificationIntervalGraphics = 0;
		else notificationIntervalGraphics = val;
	}

	public void run() {
		ControlSystemS cs;
		int i;

		while (simulationRunning) {
	                if (matchShouldStop()) stopMatch ();
			if ((simulationSteps % notificationIntervalInfo) == 0)
				performEvent (CHANGED_FOR_INFO);
			if ((simulationSteps % notificationIntervalGraphics) == 0)
				performEvent (CHANGED_FOR_GRAPHICS);
			try {
				if (matchStopped) {
					finalizeControlSystems();
					performEvent (STOPPED);
					performEvent (CHANGED_FOR_INFO);
					performEvent (CHANGED_FOR_GRAPHICS);
					synchronized(this) {
						while (matchStopped) wait();
					}
				}
			} 
			catch (InterruptedException e) {
				performEvent (CHANGED_FOR_INFO);
				performEvent (CHANGED_FOR_GRAPHICS);
			}

			takeStep ();
			yield ();
		}
		performEvent (CHANGED_FOR_INFO);
		performEvent (CHANGED_FOR_GRAPHICS);
        }

	public void takeStep () {
		SimulatedObject so;
		ControlSystemS cs;
		int i;

		currentTime = System.currentTimeMillis();
		simTimestep = currentTime - lastTime;

		/*--- run control systems and check for done ---*/
		robotsDone = true;
		for (i=0;i<controlSystems().length;i++) {
			cs = (ControlSystemS) controlSystems()[i];
			int stat = cs.takeStep();
			if (stat!=ControlSystemS.CSSTAT_OK) robotsDone = false;
		}

		/*--- run the physics ---*/
		for (i=0;i<simulatedObjects().length;i++) {
			so = simulatedObjects()[i];
			so.takeStep(simTimestep, simulatedObjects);
                }

		updateScore ();

		lastTime = currentTime;
		simulationSteps++;

	}

	public boolean matchShouldStop () {
		if (robotsDone) return false;
		else return true; 
	}

        public void finalizeControlSystems () {
		//call all the control system .quit methods
		ControlSystemS cs;
		int i;

		for (i=0;i<controlSystems().length;i++) {
			cs = controlSystems()[i];
			cs.trialEnd();
			cs.quit();
		}
         }

	public void resetMatch () {

		removeSimulatedObjects ();
		initValues ();
		addAndInitSimulatedObjects ();
		addAndInitRobots ();
		setupSimulatedObjects ();
		performEvent (CHANGED_FOR_INFO);
		performEvent (CHANGED_FOR_GRAPHICS);
        }
		

	public void removeSimulatedObjects () {
		int i;

		for (i=0;i<simulatedObjects.length;i++) {
			simulatedObjects[i] = null;
		}
		for (i=0;i<controlSystems.length;i++) {
			controlSystems[i] = null;
		}
	}

         public void stopMatch () {
                matchStopped = true;
         }

         public void stopSimulation () {
		if (matchStopped) restartMatch();
                simulationRunning = false;
         }

         public synchronized void restartMatch () {
		if (matchStopped) {
	                matchStopped = false;
		        notify();
		}
	}

	public double getTop () {
                return top;
        }

	public double getBottom () {
                return bottom;
        }

	public double getLeft () {
                return left;
        }

	public double getRight () {
                return right;
        }

	public long getCurrentTime () {
                return currentTime;
        }

	public long getSimulationSteps () {
                return simulationSteps;
        }

	public SimulatedObject[] simulatedObjects () {
                return simulatedObjects;
        }

	public ControlSystemS[] controlSystems () {
                return controlSystems;
        }

        public void addListener (SocSimListener listener) {
                simulationListener.addElement (listener);
        }

        public void removeListener (SocSimListener listener) {
                simulationListener.removeElement (listener);
        }

        public void performEvent (int id) {
                SocSimListener l;
		Enumeration enum;

                enum = simulationListener.elements();
                while (enum.hasMoreElements()) {
                        l = (SocSimListener)enum.nextElement();
                        l.socSimPerformed (id);
                }
        }

        public String toInfoString () {
		String s = new String();

		s += "---SocSim---\n";
		s += "score = " + getWestscore() + " : " + getEastscore() + "\n";
		s += "------------\n";
		s += "matchStopped = " + matchStopped + "\n";
		s += "robotsDone = " + robotsDone + "\n";
		s += "simulationRunning = " + simulationRunning + "\n";
		s += "------------\n";
		s += "simulationSteps = " + simulationSteps + "\n";
		s += "currentTime = " + new Timestamp(currentTime) + "\n";
		s += "lastTime = " + new Timestamp(lastTime) + "\n";
		s += "simTimestep = " + simTimestep + "\n";
		s += "------------\n";
		s += "listeners(count) = " + simulationListener.size() + "\n";
		s += "objects(count) = " + simulatedObjects.length + "\n";
		s += "robots(count) = " + controlSystems.length + "\n";
		s += "dimension = " + top + "," + bottom + "," + left + ","+ right + "\n"; 
		s += "intervalInfo = " + notificationIntervalInfo + "\n";
		s += "intervalGraphics = " + notificationIntervalGraphics + "\n";
		s += "seed = " + seed + "\n";

		
		return s;
        }


	private GolfBallSim getBall () {
		return (GolfBallSim) simulatedObjects[5];
	}

	private void updateScore () {
		GolfBallSim b;

		b = getBall();
		if (b.westJustScored()) westscore++;
		else if (b.eastJustScored()) eastscore++;
	}

	public int getEastscore () {
		return eastscore;
	}

	public int getWestscore () {
		return westscore;
	}


}
