package bsoc.util;

import java.util.*;

/**
* A vector which contents are sortable if they implement the
* Comparable interface.<p> 
* Sortable Vector uses a minimum search algorithm.
* @see Comparable
*/
public class SortableVector extends Vector {

	public void sort () {
		Vector sorted = new Vector();
		Enumeration enum;

		sortFromTo (this, sorted);
		removeAllElements ();
		enum = sorted.elements();
		while (enum.hasMoreElements()) {
			addElement (enum.nextElement());
		}
	}

	private static void sortFromTo (Vector from, Vector to) {
		Comparable min;

		while (from.size()>0) {
			min = searchMin (from);
			from.removeElement (min);
			to.addElement (min);
		}
	}

	private static Comparable searchMin (Vector v) {
		Comparable min, next;
		Enumeration enum;

		enum = v.elements();
		min = (Comparable)enum.nextElement();
		while (enum.hasMoreElements()) {
			next = (Comparable)enum.nextElement();
			if (min.isGreaterThan(next)) min = next;
		}
		return min;
	}
		
	/**
	* Returns true if all elements of the vector are equal.
	*/

	public boolean equals (Object o) {
                if (! (o instanceof Vector)) return false;
                else {
			Vector v = (Vector)o;
			if (v.size() != size()) return false;
			else {
				Enumeration enum1 = elements();
				Enumeration enum2 = v.elements();
				while (enum1.hasMoreElements()) {
					if (! enum1.nextElement().equals(enum2.nextElement())) return false;
				}
				return true;
			}
		}
	}

}

